@@ -3,11 +3,12 @@
 from __future__ import annotations
 import asyncio
 import json
+import logging
 from collections.abc import Callable, Iterable, Mapping
 from dataclasses import dataclass
 from datetime import UTC, datetime
 from difflib import unified_diff
-from typing import Any
+from typing import TYPE_CHECKING, Any, Protocol, runtime_checkable
 from uuid import UUID
 from orcheo.models.workflow import Workflow, WorkflowRun, WorkflowVersion
 from orcheo.triggers.cron import CronTriggerConfig
@@ -15,6 +16,10 @@ from orcheo.triggers.layer import TriggerLayer
 from orcheo.triggers.manual import ManualDispatchRequest
 from orcheo.triggers.retry import RetryDecision, RetryPolicyConfig
 from orcheo.triggers.webhook import WebhookRequest, WebhookTriggerConfig
+from orcheo.vault.oauth import CredentialHealthError, OAuthCredentialService
+
+
+logger = logging.getLogger(__name__)
 
 
 class RepositoryError(RuntimeError):
@@ -42,10 +47,187 @@ class VersionDiff:
     diff: list[str]
 
 
+@runtime_checkable
+class WorkflowRepository(Protocol):
+    """Protocol describing workflow repository behaviour."""
+
+    async def list_workflows(self, *, include_archived: bool = False) -> list[Workflow]:
+        """Return workflows stored in the repository.
+
+        Args:
+            include_archived: If True, include archived workflows. If False, only
+                return unarchived workflows. Defaults to False.
+        """
+
+    async def create_workflow(
+        self,
+        *,
+        name: str,
+        slug: str | None,
+        description: str | None,
+        tags: Iterable[str] | None,
+        actor: str,
+    ) -> Workflow:
+        """Persist and return a new workflow definition."""
+
+    async def get_workflow(self, workflow_id: UUID) -> Workflow:
+        """Return a single workflow by identifier."""
+
+    async def update_workflow(
+        self,
+        workflow_id: UUID,
+        *,
+        name: str | None,
+        description: str | None,
+        tags: Iterable[str] | None,
+        is_archived: bool | None,
+        actor: str,
+    ) -> Workflow:
+        """Mutate workflow metadata and return the updated record."""
+
+    async def archive_workflow(self, workflow_id: UUID, *, actor: str) -> Workflow:
+        """Archive the specified workflow."""
+
+    async def create_version(
+        self,
+        workflow_id: UUID,
+        *,
+        graph: dict[str, Any],
+        metadata: dict[str, Any],
+        notes: str | None,
+        created_by: str,
+    ) -> WorkflowVersion:
+        """Persist a new workflow version for the workflow."""
+
+    async def list_versions(self, workflow_id: UUID) -> list[WorkflowVersion]:
+        """Return ordered versions for the given workflow."""
+
+    async def get_version_by_number(
+        self, workflow_id: UUID, version_number: int
+    ) -> WorkflowVersion:
+        """Return a workflow version by human-friendly number."""
+
+    async def get_version(self, version_id: UUID) -> WorkflowVersion:
+        """Return a workflow version by identifier."""
+
+    async def get_latest_version(self, workflow_id: UUID) -> WorkflowVersion:
+        """Return the most recently created version for the workflow."""
+
+    async def diff_versions(
+        self, workflow_id: UUID, base_version: int, target_version: int
+    ) -> VersionDiff:
+        """Compute a diff between two workflow versions."""
+
+    async def create_run(
+        self,
+        workflow_id: UUID,
+        *,
+        workflow_version_id: UUID,
+        triggered_by: str,
+        input_payload: dict[str, Any],
+        actor: str | None = None,
+    ) -> WorkflowRun:
+        """Create a workflow run for the specified version."""
+
+    async def list_runs_for_workflow(self, workflow_id: UUID) -> list[WorkflowRun]:
+        """Return runs associated with the given workflow."""
+
+    async def get_run(self, run_id: UUID) -> WorkflowRun:
+        """Return a workflow run by identifier."""
+
+    async def mark_run_started(self, run_id: UUID, *, actor: str) -> WorkflowRun:
+        """Transition a run into the running state."""
+
+    async def mark_run_succeeded(
+        self,
+        run_id: UUID,
+        *,
+        actor: str,
+        output: dict[str, Any] | None,
+    ) -> WorkflowRun:
+        """Mark a run as successfully completed."""
+
+    async def mark_run_failed(
+        self,
+        run_id: UUID,
+        *,
+        actor: str,
+        error: str,
+    ) -> WorkflowRun:
+        """Mark a run as failed with the provided error."""
+
+    async def mark_run_cancelled(
+        self,
+        run_id: UUID,
+        *,
+        actor: str,
+        reason: str | None,
+    ) -> WorkflowRun:
+        """Cancel a run with an optional reason."""
+
+    async def reset(self) -> None:
+        """Clear all stored workflows, versions, and runs."""
+
+    async def configure_webhook_trigger(
+        self, workflow_id: UUID, config: WebhookTriggerConfig
+    ) -> WebhookTriggerConfig:
+        """Store webhook trigger configuration for a workflow."""
+
+    async def get_webhook_trigger_config(
+        self, workflow_id: UUID
+    ) -> WebhookTriggerConfig:
+        """Return the webhook trigger configuration for a workflow."""
+
+    async def handle_webhook_trigger(
+        self,
+        workflow_id: UUID,
+        *,
+        method: str,
+        headers: Mapping[str, str],
+        query_params: Mapping[str, str],
+        payload: Any,
+        source_ip: str | None,
+    ) -> WorkflowRun:
+        """Handle an inbound webhook event by enqueuing a run."""
+
+    async def configure_cron_trigger(
+        self, workflow_id: UUID, config: CronTriggerConfig
+    ) -> CronTriggerConfig:
+        """Persist cron trigger configuration for a workflow."""
+
+    async def get_cron_trigger_config(self, workflow_id: UUID) -> CronTriggerConfig:
+        """Return the cron trigger configuration for a workflow."""
+
+    async def dispatch_due_cron_runs(
+        self, *, now: datetime | None = None
+    ) -> list[WorkflowRun]:
+        """Dispatch runs for cron triggers that are due at the given time."""
+
+    async def dispatch_manual_runs(
+        self, request: ManualDispatchRequest
+    ) -> list[WorkflowRun]:
+        """Dispatch manual runs according to the provided request."""
+
+    async def configure_retry_policy(
+        self, workflow_id: UUID, config: RetryPolicyConfig
+    ) -> RetryPolicyConfig:
+        """Persist retry policy configuration for a workflow."""
+
+    async def get_retry_policy_config(self, workflow_id: UUID) -> RetryPolicyConfig:
+        """Return the retry policy configuration for a workflow."""
+
+    async def schedule_retry_for_run(
+        self, run_id: UUID, *, failed_at: datetime | None = None
+    ) -> RetryDecision | None:
+        """Return the next retry decision for the specified run if available."""
+
+
 class InMemoryWorkflowRepository:
     """Simple async-safe in-memory repository for workflows and runs."""
 
-    def __init__(self) -> None:
+    def __init__(
+        self, credential_service: OAuthCredentialService | None = None
+    ) -> None:
         """Initialize the storage containers used by the repository."""
         self._lock = asyncio.Lock()
         self._workflows: dict[UUID, Workflow] = {}
@@ -53,13 +235,24 @@ class InMemoryWorkflowRepository:
         self._versions: dict[UUID, WorkflowVersion] = {}
         self._runs: dict[UUID, WorkflowRun] = {}
         self._version_runs: dict[UUID, list[UUID]] = {}
-        self._trigger_layer = TriggerLayer()
+        self._credential_service: OAuthCredentialService | None = credential_service
+        self._trigger_layer = TriggerLayer(health_guard=credential_service)
 
-    async def list_workflows(self) -> list[Workflow]:
-        """Return all workflows stored within the repository."""
+    async def list_workflows(self, *, include_archived: bool = False) -> list[Workflow]:
+        """Return workflows stored within the repository.
+
+        Args:
+            include_archived: If True, include archived workflows. If False, only return
+                unarchived workflows. Defaults to False.
+
+        Returns:
+            List of workflows matching the filter criteria.
+        """
         async with self._lock:
             return [
-                workflow.model_copy(deep=True) for workflow in self._workflows.values()
+                workflow.model_copy(deep=True)
+                for workflow in self._workflows.values()
+                if include_archived or not workflow.is_archived
             ]
 
     async def create_workflow(
@@ -273,6 +466,8 @@ class InMemoryWorkflowRepository:
             if workflow_id not in self._workflows:
                 raise WorkflowNotFoundError(str(workflow_id))
 
+            await self._ensure_workflow_health(workflow_id, actor=actor or triggered_by)
+
             run = self._create_run_locked(
                 workflow_id=workflow_id,
                 workflow_version_id=workflow_version_id,
@@ -451,6 +646,8 @@ class InMemoryWorkflowRepository:
             if version is None:
                 raise WorkflowVersionNotFoundError(str(latest_version_id))
 
+            await self._ensure_workflow_health(workflow_id, actor="webhook")
+
             request = WebhookRequest(
                 method=method,
                 headers=headers,
@@ -501,6 +698,16 @@ class InMemoryWorkflowRepository:
                 workflow_id = plan.workflow_id
                 if workflow_id not in self._workflows:
                     continue
+                try:
+                    await self._ensure_workflow_health(workflow_id, actor="cron")
+                except CredentialHealthError as exc:
+                    logger.warning(
+                        "Skipping cron dispatch for workflow %s "
+                        "due to credential health error: %s",
+                        workflow_id,
+                        exc,
+                    )
+                    continue
                 version_ids = self._workflow_versions.get(workflow_id)
                 if not version_ids:
                     continue
@@ -543,6 +750,10 @@ class InMemoryWorkflowRepository:
             triggered_by = plan.triggered_by
             resolved_runs = plan.runs
 
+            await self._ensure_workflow_health(
+                request.workflow_id, actor=plan.actor or triggered_by
+            )
+
             for resolved in resolved_runs:
                 version = self._versions.get(resolved.workflow_version_id)
                 if version is None or version.workflow_id != request.workflow_id:
@@ -561,6 +772,16 @@ class InMemoryWorkflowRepository:
                 runs.append(run.model_copy(deep=True))
             return runs
 
+    async def _ensure_workflow_health(
+        self, workflow_id: UUID, *, actor: str | None = None
+    ) -> None:
+        service = self._credential_service
+        if service is None:
+            return
+        report = await service.ensure_workflow_health(workflow_id, actor=actor)
+        if not report.is_healthy:
+            raise CredentialHealthError(report)
+
     async def configure_retry_policy(
         self, workflow_id: UUID, config: RetryPolicyConfig
     ) -> RetryPolicyConfig:
@@ -587,8 +808,25 @@ class InMemoryWorkflowRepository:
             return self._trigger_layer.next_retry_for_run(run_id, failed_at=failed_at)
 
 
+if TYPE_CHECKING:  # pragma: no cover - import for typing only
+    from orcheo_backend.app.repository_sqlite import SqliteWorkflowRepository
+
+
+def __getattr__(name: str) -> Any:
+    """Provide lazy access to optional repository implementations."""
+    if name == "SqliteWorkflowRepository":
+        from orcheo_backend.app.repository_sqlite import (
+            SqliteWorkflowRepository as _SqliteWorkflowRepository,
+        )
+
+        return _SqliteWorkflowRepository
+    raise AttributeError(name)
+
+
 __all__ = [
+    "WorkflowRepository",
     "InMemoryWorkflowRepository",
+    "SqliteWorkflowRepository",
     "RepositoryError",
     "VersionDiff",
     "WorkflowNotFoundError",
